@@ -2,7 +2,7 @@
 
 package Git::Hooks::CheckCommit;
 # ABSTRACT: Git::Hooks plugin to enforce commit policies
-$Git::Hooks::CheckCommit::VERSION = '1.9.0';
+$Git::Hooks::CheckCommit::VERSION = '1.10.0';
 use 5.010;
 use utf8;
 use strict;
@@ -126,7 +126,7 @@ sub _canonical_identity {
             chomp($cache->{canonical}{$identity} = $canonical);
         } otherwise {
             $cache->{canonical}{$identity} = $identity;
-            $git->error($PKG, <<EOS);
+            $git->error($PKG, <<'EOS');
 The githooks.checkcommit.canonical option requires the git-check-mailmap
 command which isn't found. It's available since Git 1.8.4. You should either
 upgrade your Git or disable this option.
@@ -329,7 +329,7 @@ Git::Hooks::CheckCommit - Git::Hooks plugin to enforce commit policies
 
 =head1 VERSION
 
-version 1.9.0
+version 1.10.0
 
 =head1 DESCRIPTION
 
