@@ -2,7 +2,7 @@
 
 package Git::Hooks::CheckFile;
 # ABSTRACT: Git::Hooks plugin for checking files
-$Git::Hooks::CheckFile::VERSION = '1.9.0';
+$Git::Hooks::CheckFile::VERSION = '1.10.0';
 use 5.010;
 use utf8;
 use strict;
@@ -11,20 +11,60 @@ use Git::Hooks qw/:DEFAULT :utils/;
 use Data::Util qw(:check);
 use Text::Glob qw/glob_to_regex/;
 use Path::Tiny;
+use List::MoreUtils qw/any none/;
 use Error qw(:try);
 
 my $PKG = __PACKAGE__;
 (my $CFG = __PACKAGE__) =~ s/.*::/githooks./;
 
+sub check_command {
+    my ($git, $commit, $file, $command) = @_;
+
+    my $tmpfile = $git->blob($commit, $file);
+    $tmpfile or return;
+
+    # interpolate filename in $command
+    (my $cmd = $command) =~ s/\{\}/\'$tmpfile\'/g;
+
+    # execute command and update $errors
+    my $saved_output = redirect_output();
+    my $exit = system $cmd;
+    my $output = restore_output($saved_output);
+    if ($exit != 0) {
+        $command =~ s/\{\}/\'$file\'/g;
+        my $message = do {
+            if ($exit == -1) {
+                "command '$command' could not be executed: $!";
+            } elsif ($exit & 127) {
+                sprintf("command '%s' was killed by signal %d, %s coredump",
+                        $command, ($exit & 127), ($exit & 128) ? 'with' : 'without');
+            } else {
+                sprintf("command '%s' failed with exit code %d", $command, $exit >> 8);
+            }
+        };
+
+        # Replace any instance of the $tmpfile name in the output by
+        # $file to avoid confounding the user.
+        $output =~ s/\Q$tmpfile\E/$file/g;
+
+        $git->error($PKG, $message, $output);
+        return;
+    } else {
+        # FIXME: What we should do with eventual output from a
+        # successful command?
+    }
+    return 1;
+}
+
 sub check_new_files {
     my ($git, $commit, @files) = @_;
 
     return 1 unless @files;     # No new file to check
 
     # Construct a list of command checks from the
-    # githooks.checkfile.basename configuration. Each check in the list is a
+    # githooks.checkfile.name configuration. Each check in the list is a
     # pair containing a regex and a command specification.
-    my @checks;
+    my @name_checks;
     foreach my $check ($git->get_config($CFG => 'name')) {
         my ($pattern, $command) = split / /, $check, 2;
         if ($pattern =~ m/^qr(.)(.*)\g{1}/) {
@@ -33,18 +73,42 @@ sub check_new_files {
             $pattern = glob_to_regex($pattern);
         }
         $command .= ' {}' unless $command =~ /\{\}/;
-        push @checks, [$pattern => $command];
+        push @name_checks, [$pattern => $command];
     }
 
     # See if we have to check a file size limit
     my $sizelimit = $git->get_config($CFG => 'sizelimit');
 
+    # Grok all REGEXP checks
+    my %re_checks;
+    foreach my $name (qw/basename path/) {
+        foreach my $check (qw/deny allow/) {
+            $re_checks{$name}{$check} = [map {qr/$_/} $git->get_config("$CFG.$name" => $check)];
+        }
+    }
+
     # Now we iterate through every new file and apply to them the matching
     # commands.
     my $errors = 0;
 
   FILE:
     foreach my $file (@files) {
+        my $basename = path($file)->basename;
+
+        if (any  {$basename =~ $_} @{$re_checks{basename}{deny}} and
+            none {$basename =~ $_} @{$re_checks{basename}{allow}}) {
+            $git->error($PKG, "File '$file' basename was denied.");
+            ++$errors;
+            next FILE;          # Don't botter checking the contents of invalid files
+        }
+
+        if (any  {$file =~ $_} @{$re_checks{path}{deny}} and
+            none {$file =~ $_} @{$re_checks{path}{allow}}) {
+            $git->error($PKG, "File '$file' path was denied.");
+            ++$errors;
+            next FILE;          # Don't botter checking the contents of invalid files
+        }
+
         my $size = $git->file_size($commit, $file);
         if ($sizelimit && $sizelimit < $size) {
             $git->error($PKG, "File '$file' has $size bytes but the current limit is just $sizelimit bytes.");
@@ -52,43 +116,9 @@ sub check_new_files {
             next FILE;    # Don't botter checking the contents of huge files
         }
 
-        my $basename = path($file)->basename;
-      COMMAND:
-        foreach my $command (map {$_->[1]} grep {$basename =~ $_->[0]} @checks) {
-            my $tmpfile = $git->blob($commit, $file)
-                or ++$errors
-                    and next COMMAND;
-
-            # interpolate filename in $command
-            (my $cmd = $command) =~ s/\{\}/\'$tmpfile\'/g;
-
-            # execute command and update $errors
-            my $saved_output = redirect_output();
-            my $exit = system $cmd;
-            my $output = restore_output($saved_output);
-            if ($exit != 0) {
-                $command =~ s/\{\}/\'$file\'/g;
-                my $message = do {
-                    if ($exit == -1) {
-                        "command '$command' could not be executed: $!";
-                    } elsif ($exit & 127) {
-                        sprintf("command '%s' was killed by signal %d, %s coredump",
-                                $command, ($exit & 127), ($exit & 128) ? 'with' : 'without');
-                    } else {
-                        sprintf("command '%s' failed with exit code %d", $command, $exit >> 8);
-                    }
-                };
-
-                # Replace any instance of the $tmpfile name in the output by
-                # $file to avoid confounding the user.
-                $output =~ s/\Q$tmpfile\E/$file/g;
-
-                $git->error($PKG, $message, $output);
-                ++$errors;
-            } else {
-                # FIXME: What we should do with eventual output from a
-                # successful command?
-            }
+        foreach my $command (map {$_->[1]} grep {$basename =~ $_->[0]} @name_checks) {
+            check_command($git, $commit, $file, $command)
+                or ++$errors;
         }
     }
 
@@ -148,7 +178,7 @@ Git::Hooks::CheckFile - Git::Hooks plugin for checking files
 
 =head1 VERSION
 
-version 1.9.0
+version 1.10.0
 
 =head1 DESCRIPTION
 
@@ -177,7 +207,7 @@ option:
 
     git config --add githooks.plugin CheckFile
 
-=for Pod::Coverage check_new_files check_affected_refs check_commit check_patchset
+=for Pod::Coverage check_command check_new_files check_affected_refs check_commit check_patchset
 
 =head1 NAME
 
@@ -237,6 +267,35 @@ repository. If set explicitly to 0 (zero), no limit is imposed, which is the
 same as not specifying it. But it can be useful to override a global
 specification in a particular repository.
 
+=head2 githooks.checkfile.basename.deny REGEXP
+
+This directive denies files which basenames match REGEXP.
+
+=head2 githooks.checkfile.basename.allow REGEXP
+
+This directive allows files which basenames match REGEXP. Since by default
+all basenames are allowed this directive is useful only to prevent a
+B<githooks.checkfile.basename.deny> directive to deny the same basename.
+
+The basename checks are evaluated so that a file is denied only if it's
+basename matches any B<basename.deny> directive and none of the
+B<basename.allow> directives.  So, for instance, you would apply it like
+this to allow the versioning of F<.gitignore> file while denying any other
+file with a name beginning with a dot.
+
+    [githooks "checkfile"]
+        basename.allow ^\\.gitignore
+        basename.deny  ^\\.
+
+=head2 githooks.checkfile.path.deny REGEXP
+
+This directive denies files which full paths match REGEXP.
+
+=head2 githooks.checkfile.path.allow REGEXP
+
+This directive allows files which full paths match REGEXP. It's useful in
+the same way that B<githooks.checkfile.basename.deny> is.
+
 =head1 AUTHOR
 
 Gustavo L. de M. Chaves <gnustavo@cpan.org>
